import ipaddress

from orchestrator import workflow
from orchestrator.forms import FormPage
from orchestrator.forms.validators import Choice, UniqueConstrainedList
from orchestrator.targets import Target
from orchestrator.types import FormGenerator, State, SubscriptionLifecycle
from orchestrator.workflow import StepList, done, init, step
from orchestrator.workflows.steps import resync, set_status, store_process_subscription

from gso.products import ProductType
from gso.products.product_blocks.iptrunk import IptrunkType, PhyPortCapacity
from gso.products.product_types.iptrunk import IptrunkInactive, IptrunkProvisioning
from gso.services import subscriptions
from gso.services.crm import get_customer_by_name
from gso.utils.helpers import LAGMember
from gso.workflows.iptrunk.create_iptrunk import initialize_subscription


def _generate_routers() -> dict[str, str]:
    """Generate a dictionary of router IDs and descriptions."""
    routers = {}
    for subscription in subscriptions.get_active_router_subscriptions(includes=["subscription_id", "description"]):
        routers[str(subscription["subscription_id"])] = subscription["description"]

    return routers


def initial_input_form_generator() -> FormGenerator:
    routers = _generate_routers()
    router_enum = Choice("Select a router", zip(routers.keys(), routers.items(), strict=True))  # type: ignore[arg-type]

    class CreateIptrunkForm(FormPage):
        class Config:
            title = "Import Iptrunk"

        customer: str
        geant_s_sid: str
        iptrunk_description: str
        iptrunk_type: IptrunkType
        iptrunk_speed: PhyPortCapacity
        iptrunk_minimum_links: int

        side_a_node_id: router_enum  # type: ignore[valid-type]
        side_a_ae_iface: str
        side_a_ae_geant_a_sid: str
        side_a_ae_members: UniqueConstrainedList[LAGMember]

        side_b_node_id: router_enum  # type: ignore[valid-type]
        side_b_ae_iface: str
        side_b_ae_geant_a_sid: str
        side_b_ae_members: UniqueConstrainedList[LAGMember]

        iptrunk_ipv4_network: ipaddress.IPv4Network
        iptrunk_ipv6_network: ipaddress.IPv6Network

    initial_user_input = yield CreateIptrunkForm

    return initial_user_input.dict()


@step("Create a new subscription")
def create_subscription(customer: str) -> State:
    customer_id = get_customer_by_name(customer)["id"]
    product_id = subscriptions.get_product_id_by_name(ProductType.IP_TRUNK)
    subscription = IptrunkInactive.from_product_id(product_id, customer_id)

    return {
        "subscription": subscription,
        "subscription_id": subscription.subscription_id,
    }


@step("Update IPAM Stub for Subscription")
def update_ipam_stub_for_subscription(
    subscription: IptrunkProvisioning,
    iptrunk_ipv4_network: ipaddress.IPv4Network,
    iptrunk_ipv6_network: ipaddress.IPv6Network,
) -> State:
    subscription.iptrunk.iptrunk_ipv4_network = iptrunk_ipv4_network
    subscription.iptrunk.iptrunk_ipv6_network = iptrunk_ipv6_network

    return {"subscription": subscription}


@workflow(
    "Import iptrunk",
    initial_input_form=initial_input_form_generator,
    target=Target.SYSTEM,
)
def import_iptrunk() -> StepList:
    return (
        init
        >> create_subscription
        >> store_process_subscription(Target.CREATE)
        >> initialize_subscription
        >> update_ipam_stub_for_subscription
        >> set_status(SubscriptionLifecycle.ACTIVE)
        >> resync
        >> done
    )
